from checkov.graph.terraform.checks_infra.solvers.attribute_queries.any_attribute_solver import AnyResourceSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.array_contains_attribute_solver import ArrayContainsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.array_not_contains_attribute_solver import ArrayNotContainsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.ending_with_attribute_solver import EndingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.equals_attribute_solver import EqualsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.exists_attribute_solver import ExistsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.not_ending_with_attribute_solver import NotEndingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.not_equals_attribute_solver import NotEqualsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.not_exists_attribute_solver import NotExistsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.not_starting_with_attribute_solver import NotStartingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.starting_with_attribute_solver import StartingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_queries.within_attribute_solver import WithinAttributeSolver